import unittest

from mr.monster.rewrite import RewriteFactory

def PathAssertionEndpoint(expected, test):
    def _(environ, start_response):
        test.assertEqual(environ['PATH_INFO'], expected)
        if environ['PATH_INFO'] == expected:
            start_response("200 OK", [])
        return []
    return _

class response(object):
    def start_response(self, status, headers):
        self.status = status
        self.headers = headers

class test_urls(unittest.TestCase):
    
    def test_autodetect_if_no_options(self):
        factory = RewriteFactory({})
        r = response()
        app = factory(PathAssertionEndpoint("/VirtualHostBase/http/127.0.0.1:8080/VirtualHostRoot/", self))
        app({"SERVER_NAME":"127.0.0.1",
             "SERVER_PORT":"8080",
             "REQUEST_METHOD":"GET",
             "PATH_INFO":"/"}, r.start_response)
        assert r.status.startswith("200")
        
    def test_fail_if_host_provided_but_not_port(self):
        self.assertRaises(AttributeError, RewriteFactory, object(), host="www.example.com")

    def test_vhm_on_host_and_port(self):
        factory = RewriteFactory({}, host="www.example.com", port="80")
        r = response()
        app = factory(PathAssertionEndpoint("/VirtualHostBase/http/www.example.com:80/VirtualHostRoot/", self))
        app({"REQUEST_METHOD":"GET",
             "PATH_INFO":"/"}, r.start_response)
        assert r.status.startswith("200")

    def test_fail_if_port_provided_but_not_host(self):
        self.assertRaises(AttributeError, RewriteFactory, object(), port="80")
    
    def test_vhm_host_autodetect(self):
        factory = RewriteFactory({}, autodetect="true")
        r = response()
        app = factory(PathAssertionEndpoint("/VirtualHostBase/http/127.0.0.1:8080/VirtualHostRoot/", self))
        app({"SERVER_NAME":"127.0.0.1",
             "SERVER_PORT":"8080",
             "REQUEST_METHOD":"GET",
             "PATH_INFO":"/"}, r.start_response)
        assert r.status.startswith("200")

    def test_vhm_host_autodetect_http1_1_no_port(self):
        factory = RewriteFactory({}, autodetect="true")
        r = response()
        app = factory(PathAssertionEndpoint("/VirtualHostBase/http/www.example.com:80/VirtualHostRoot/", self))
        app({"HTTP_HOST":"www.example.com",
             "SERVER_NAME":"127.0.0.1",
             "SERVER_PORT":"8080",
             "REQUEST_METHOD":"GET",
             "PATH_INFO":"/"}, r.start_response)
        assert r.status.startswith("200")

    def test_vhm_host_autodetect_http1_1_explicit_port(self):
        factory = RewriteFactory({}, autodetect="true")
        r = response()
        app = factory(PathAssertionEndpoint("/VirtualHostBase/http/www.example.com:8080/VirtualHostRoot/", self))
        app({"HTTP_HOST":"www.example.com:8080",
             "SERVER_NAME":"127.0.0.1",
             "SERVER_PORT":"8080",
             "REQUEST_METHOD":"GET",
             "PATH_INFO":"/"}, r.start_response)
        assert r.status.startswith("200")


    def test_double_slash(self):
        factory = RewriteFactory({}, host="www.example.com", port="80", internalpath="/foo",externalpath="/supersite/")
        r = response()
        app = factory(PathAssertionEndpoint("/VirtualHostBase/http/www.example.com:80/foo/VirtualHostRoot/_vh_supersite", self))
        app({"REQUEST_METHOD":"GET",
             "PATH_INFO":"/supersite"}, r.start_response)
        assert r.status.startswith("200")


    def test_root_exception(self):
        factory = RewriteFactory({}, host="www.example.com", port="80", internalpath="/foo",externalpath="/")
        r = response()
        app = factory(PathAssertionEndpoint("/VirtualHostBase/http/www.example.com:80/foo/VirtualHostRoot/", self))
        app({"REQUEST_METHOD":"GET",
             "PATH_INFO":"/"}, r.start_response)
        assert r.status.startswith("200")

    def test_folder_on_zope_side(self):
        factory = RewriteFactory({}, host="www.example.com", port="80", internalpath="/foo")
        r = response()
        app = factory(PathAssertionEndpoint("/VirtualHostBase/http/www.example.com:80/foo/VirtualHostRoot/", self))
        app({"REQUEST_METHOD":"GET",
             "PATH_INFO":"/"}, r.start_response)
        assert r.status.startswith("200")

    def test_folder_on_client_side(self):
        factory = RewriteFactory({}, host="www.example.com", port="80", externalpath="/bar")
        r = response()
        app = factory(PathAssertionEndpoint("/VirtualHostBase/http/www.example.com:80/VirtualHostRoot/_vh_bar", self))
        app({"REQUEST_METHOD":"GET",
             "PATH_INFO":"/bar"}, r.start_response)
        assert r.status.startswith("200")

    def test_accessing_resource(self):
        factory = RewriteFactory({}, host="www.example.com", port="80")
        r = response()
        app = factory(PathAssertionEndpoint("/VirtualHostBase/http/www.example.com:80/VirtualHostRoot/my/thing/is/cool", self))
        app({"REQUEST_METHOD":"GET",
             "PATH_INFO":"/my/thing/is/cool"}, r.start_response)
        assert r.status.startswith("200")

    def test_accessing_script(self):
        factory = RewriteFactory({}, host="www.example.com", port="80")
        r = response()
        app = factory(PathAssertionEndpoint("/VirtualHostBase/http/www.example.com:80/VirtualHostRoot/isay/my/thing/is/cool", self))
        app({"REQUEST_METHOD":"GET",
             "SCRIPT_NAME":"/isay",
             "PATH_INFO":"/my/thing/is/cool"}, r.start_response)
        assert r.status.startswith("200")


import re
from webob import Request, Response

def RewriteFactory(global_config, **local_conf):
    if "host" in local_conf:
        if "port" not in local_conf:
            raise AttributeError("You must also supply a port")

    if "port" in local_conf:
        if "host" not in local_conf:
            raise AttributeError("You must also supply a host")

    
    def factory(app):
        return RewriteMiddleware(app, **local_conf)
    return factory

_marker = object()

hostre = re.compile("(.*):(\d+)")

class RewriteMiddleware(object):
    """An endpoint"""
    
    def __init__(self, app, host=None, 
                            port=None, 
                            internalpath='', 
                            externalpath='', 
                            autodetect=_marker):
        self.host = host
        self.port = port
        self.internalpath = internalpath.split("/")

        # Support special case for root external path.
        if externalpath.endswith('/'):
            externalpath = externalpath[:-1]
        self.externalpath = externalpath.split("/")
        
        if autodetect is _marker and self.host is None and self.port is None:
            self.autodetect = True
        else:
            self.autodetect = str(autodetect).lower() == "true"
            
        self.app = app
    
    def __call__(self, environ, start_response):
        
        options = {"host": self.host, 
                   "port": self.port, 
                   "inpath":"/".join(self.internalpath),
                   "outpath":"/_vh_".join(self.externalpath)}
        
        if self.autodetect:
            host = environ.get('HTTP_HOST', None)
            if host is not None:
                # Using HTTP 1.1
                parsed = hostre.search(host)
                if parsed:
                    parsed = parsed.groups()
                    options['host'] = parsed[0]
                    if options['port'] is None:
                        options['port'] = parsed[1]
                else:
                    options['host'] = host
                    if options['port'] is None:
                        options['port'] = '80'
            else:
                # HTTP 1.0 or 0.9
                host = environ['SERVER_NAME']
                options['host'] = host
                if options['port'] is None:
                    options['port'] = environ['SERVER_PORT']
        
    
        if "SCRIPT_NAME" in environ:
            options['PATH_INFO'] = environ['SCRIPT_NAME']
            environ['SCRIPT_NAME'] = ''
        else:
            options['PATH_INFO'] = ''
        
        options['PATH_INFO'] += environ['PATH_INFO']
        
        op = '/'.join(self.externalpath)
        if options['PATH_INFO'].startswith(op):
            pl = len(op)
            options['PATH_INFO'] = options['PATH_INFO'][pl:]
        
        format = "/VirtualHostBase/http/"   \
                 "%(host)s:%(port)s"        \
                 "%(inpath)s"              \
                 "/VirtualHostRoot"         \
                 "%(outpath)s"             \
                 "%(PATH_INFO)s"    % options
        
        if options['host'] is not None:
            environ['PATH_INFO'] = format
        
        request = Request(environ)
        response = request.get_response(self.app)
        return response(environ, start_response)



from setuptools import setup, find_packages
import os

version = '1.3'

setup(name='mr.monster',
      version=version,
      description="A URL rewriting middleware that emulates a VHM url.",
      long_description=open(os.path.join('src', 'mr', 'monster', 'README.txt')).read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='Matthew Wilkes',
      author_email='matt.wilkes@teamrubber.com',
      url='http://www.teamrubber.com',
      license='BSD',
      package_dir = {'':'src'},
      packages=find_packages('src', exclude=['ez_setup']),
      namespace_packages=['mr'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'WebOb',
      ],
      entry_points="""
      # -*- Entry points: -*-
[paste.filter_factory]
rewrite = mr.monster.rewrite:RewriteFactory
      """,
      )
